/**
 * Hilfsklasse zum Einlesen von Kommandozeilen-Parametern
 * fr Java-Programme.
 *
 * Die Parameter-Namen mssen mit einem '-' (Hyphen,
 * Bindestrich) beginnen.
 *
 * Leerzeichen innerhalb eines Parameter-Values sind mglich.
 * Dabei darf aber kein Teilstck mit einem '-' (Hyphen,
 * Bindestrich) und dem nchsten Parameter-Namen beginnen.
 *
 * Durch das Einschliessen der Parameter in '"' (Quota,
 * Anfhrungstriche) sind auch mehrere Spaces (Leerzeichen)
 * hintereinander in den Parametern mglich.
 *
 * Alle Parameter mssen in der geforderten Reihenfolge
 * vorhanden sein.
 *
 * Das mitgelieferte Beispiel hat die Parameter prm1 bis prm3.
 *
 * Fr die eigene Anwendung bitte den Code entsprechend
 * anpassen.
 *
 * @author Heiner K&uuml;cker
 */
public class CommandLineParamReader
{

    /** Die Namen der Kommandozeilen-Parameter */
    final static String[] OPTION_NAMES_ARR =
        new String[] { "prm1", "prm2", "prm3", };

    /** Fehlermeldung fr fehlende Optionen */
    final static String PARAM_MISSED_ERR_TXT =
        "options " + Util.arr2Str(OPTION_NAMES_ARR, " ", "-") + " expected";

    /** der gesamte Parameter-String fr Debug-Zwecke */
    private String prmStr;

    /** Parameter 1 */
    private String prm1;

    /** Parameter 2 */
    private String prm2;

    /** Parameter 3 */
    private String prm3;

    /**
     * @return Returns the prmStr.
     */
    public String getPrmStr()
    {
        return prmStr;
    }

    /**
     * @return
     */
    public static String getPARAM_MISSED_ERR_TXT()
    {
        return PARAM_MISSED_ERR_TXT;
    }

    /**
     * @return
     */
    public String getPrm1()
    {
        return prm1;
    }

    /**
     * @return
     */
    public String getPrm2()
    {
        return prm2;
    }

    /**
     * @return
     */
    public String getPrm3()
    {
        return prm3;
    }

    /**
     * Konstruktor private,
     * Instanziierung nur aus
     * {@link #parse(String)}.
     */
    private CommandLineParamReader()
    {
    }

    /**
     * Parsen der Kommandozeile als String-Array
     * und zurckgeben eines {@link CommandLineParamReader}-Objekts.
     *
     * @param pStrArr Java-Kommandozeilen-Parameter
     * @return erzeugtes Parameter-Objekt
     */
    public static CommandLineParamReader parse(final String[] pStrArr)
    {
        final String cmdLineStr = Util.fillGapsWithDelim(pStrArr, " ");
        if (Util.isEmpty(cmdLineStr))
        {
            throw new IllegalArgumentException("command line parameter missed");
        }
        final String[] optValueArr = readParams(cmdLineStr, OPTION_NAMES_ARR);

        final CommandLineParamReader params = new CommandLineParamReader();
        params.prmStr = cmdLineStr;
        params.prm1 = optValueArr[0];
        params.prm2 = optValueArr[1];
        params.prm3 = optValueArr[2];
        return params;
    }

    /**
     * Debug-Ausgabe
     */
    public String toString()
    {
        return this.getClass().getName()
            + "[\n prmStr: "
            + prmStr
            + "\n prm1: "
            + prm1
            + "\n prm2: "
            + prm2
            + "\n prm3: "
            + prm3
            + "\n]";
    }

    /**
     * Einlesen der Kommandozeilen-Parameter anhand
     * der bergebenen Options-Namen.
     *
     * @param str Kommandozeilen-String
     * @param option_names_arr2 Options-Namen als String-Array
     * @return Options-Values als String-Array
     */
    private static String[] readParams(
        final String pCmdLineStr,
        final String[] pOptionNamesStrArr)
    {
        String runCmdLineStr = pCmdLineStr;
        final String[] optionValuesStrArr =
            new String[pOptionNamesStrArr.length];
        //Schleife ber die Options-Namen
        for (int i = 0; i < pOptionNamesStrArr.length; i++)
        {
            // Zusammenbau Option-Name mit fhrendem Bindestrich und nachfolgendem Space
            final String optionPreStr = "-" + pOptionNamesStrArr[i] + " ";
            //Test auf Vorhandensein der Option
            if (!runCmdLineStr.startsWith(optionPreStr))
            {
                throw new IllegalArgumentException(
                    "command line parameter '" + optionPreStr + "' missed");
            }
            //Option-Prefix abschneiden
            runCmdLineStr = Util.cutLeft(runCmdLineStr, optionPreStr);

            //Option-Value ermitteln
            final String optionValueStr;
            if (i >= pOptionNamesStrArr.length - 1)
            {
                //letzter Durchlauf
                optionValueStr = runCmdLineStr;
                //Option-Value vermerken
                optionValuesStrArr[i] = optionValueStr;
            }
            else
            {
                //nicht letzter Durchlauf
                //Suche bis zur nchsten Option
                final String nextOptionPreStr =
                    " -" + pOptionNamesStrArr[i + 1] + " ";
                int optValueEndPos = runCmdLineStr.indexOf(nextOptionPreStr);
                if (optValueEndPos < 0)
                {
                    //nicht gefunden
                    optValueEndPos = runCmdLineStr.length();
                }
                optionValueStr = runCmdLineStr.substring(0, optValueEndPos);
                //Option-Value vermerken
                optionValuesStrArr[i] = optionValueStr;
                //Option-Value abschneiden
                runCmdLineStr = Util.cutLeft(runCmdLineStr, optionValueStr);
                runCmdLineStr = Util.cutLeft(runCmdLineStr, " ");
            }
        }
        return optionValuesStrArr;
    }

    /**
     * Test.
     *
     * @param args
     */
    public static void main(String[] args)
    {
        CommandLineParamReader params =
            parse(
                Util.split(
                    "-prm1 value1 -prm2 value2 -prm3 Test the CommandLineParamReader",
                    " "));
        System.out.println("params: " + params);
    }
}
